/*
** Module   : MenuGuiImpl.java
** Abstract : GUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150525 Created initial version.
*/
package com.goldencode.p2j.ui.client.gui;

import java.awt.event.MouseEvent;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.SubMenuChuiImpl;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;

/**
 * <code>Menu</code> GUI client implementation class. 
 */
public class MenuGuiImpl
extends Menu<GuiOutputManager>
{
   /** Screen driver */
   private GuiDriver gd;

   /** Font resolver */
   private GuiFontResolver gf;
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuGuiImpl(WidgetId id, MenuConfig cfg)
   {
      super(id, cfg);
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, MenuConfig cfg)
   {
      super.initialize(id, cfg);
      
      gd = (GuiDriver) OutputManager.getDriver();
      
      Widget<?> parent = ThinClient.getInstance().getWidget(cfg.frameId);
      Window<?> window = parent.window();
      
      gf = new GuiFontResolver(window, gd, config(), false);
   }
   
   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      final GuiColorResolver gce = new GuiColorResolver(gd, config(), 
               SysColor.COLOR_MENU, 
               SysColor.COLOR_BTNTEXT);
      gce.colors();

      final NativePoint p = physicalLocation();
      final NativeDimension dim = physicalDimension();
      NativeRectangle clip = clipRectangle(p, dim);
      
      gd.draw(p, clip, new Runnable()
      {
         @Override
         public void run()
         {
            Window<?> window = Window.resolveWindow(MenuGuiImpl.this);
            int fh = FontManager.getFontHeight(window, gf.resolveFontNum(), gd);
            gd.setColor(gce.bgColor);
            gd.fillRect(0, 0, dim.width, fh + 5 + 5);

            MenuGuiImpl.super.draw();
         }
      });
   }
   
   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    */
   @Override
   public Dimension dimension()
   {
      return dimension(true);
   }
   
   /**
    * Lay out container. Set children locations.
    */
   @Override
   public void doLayout()
   {
      super.doLayout();
      
      TitledWindow<?> wnd = this.ancestor();
      CoordinatesConversion c = this.ancestor().screen().coordinates();
      int itemHeight = FontManager.getFontHeight(wnd, gf.resolveFontNum(), gd);
      itemHeight += 5 + 5; // borders
      
      Widget<GuiOutputManager>[] w = widgets();
      int x = 0; // this.physicalLocation.x;
      int y = 0; //this.physicalLocation.y;
      
      for (int i = 0; i < w.length; i++)
      {
         if (!config().popupOnly) // menu-bar
         {
            w[i].setPhysicalLocation(x, y);
            w[i].setLocation(c.widthFromNative(x), c.heightFromNative(y));
            x += c.widthToNative(w[i].width()); 
         }
         else
         {
            w[i].setPhysicalLocation(x, y + i * itemHeight); 
            w[i].setLocation(c.widthFromNative(x), c.heightFromNative(y + i * itemHeight));
         }
         
         if(w[i] instanceof SubMenu)
         {
            ((SubMenu<?>)w[i]).doLayout();
         }
      }
   }
   
   /**
    * Notification of a mouse clicked event occurred for this widget.  No-op by default.
    * 
    * @param    e
    *           The mouse event.
    */
   @Override
   public void mouseClicked(MouseEvent e)
   {
      // TODO
   }

   /**
    * Calculates MENU location relative to its parent if it is POPUP-ONLY or 
    * [0, 0] if it is MENUBAR.
    * 
    * @return   point of location relative to its parent.
    */
   private Point calcLocation()
   {
      if (config().popupOnly)
      {
         Widget<?> owner = screen().getRegistry().getComponent(config().ownerId);
         if (owner != null)
         {
            AbstractWidget<?> awOwner = (AbstractWidget<?>) owner;
            Point ownloc = awOwner.location();
            
            // pop-up menu is shown right under owner, shifted by 1 horizontally 
            return new Point(ownloc.x + 1, ownloc.y + awOwner.height() + 1);
         }
      }
      
      return new Point(0, 0); // menu-bar is always on top
   }
   
   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    * 
    * @param   includeNested
    *          <code>true</code> if we should process nested sub-menus
    */
   private Dimension dimension(boolean includeNested)
   {
      double height = SubMenuChuiImpl.getMaxItemHeight(this, includeNested);
      double width = SubMenuGuiImpl.getMaxSubMenuHeight(this, includeNested);
      
      if (!config().popupOnly) // menubar
      {
         width = this.width();
      }
      
      return new Dimension(width, height); // left and right borders with insets
   }

}
